@@ -1,13 +1,20 @@
+from typing import Dict, List, Tuple, TypeVar, Optional
+
 import redis
 
 from .. import app_settings
 from ..job import Job
-from .base import BaseBackend
+from .base import BackendWithDeduplicate
+from ..types import QueueName, WorkerNumber
 from ..utils import get_worker_numbers
-from ..progress_logger import NULL_PROGRESS_LOGGER
+from ..progress_logger import ProgressLogger, NULL_PROGRESS_LOGGER
+
+# Work around https://github.com/python/mypy/issues/9914. Name needs to match
+# that in progress_logger.py.
+T = TypeVar('T')
 
 
-class ReliableRedisBackend(BaseBackend):
+class ReliableRedisBackend(BackendWithDeduplicate):
     """
     This backend manages a per-queue-per-worker 'processing' queue. E.g. if we
     had a queue called 'django_lightweight_queue:things', and two workers, we
@@ -29,13 +36,13 @@ class ReliableRedisBackend(BaseBackend):
     This backend has at-least-once semantics.
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.client = redis.StrictRedis(
             host=app_settings.REDIS_HOST,
             port=app_settings.REDIS_PORT,
         )
 
-    def startup(self, queue):
+    def startup(self, queue: QueueName) -> None:
         main_queue_key = self._key(queue)
 
         pattern = self._prefix_key(
@@ -53,7 +60,7 @@ class ReliableRedisBackend(BaseBackend):
         )
         processing_queue_keys = current_processing_queue_keys - expected_processing_queue_keys
 
-        def move_processing_jobs_to_main(pipe):
+        def move_processing_jobs_to_main(pipe: redis.client.Pipeline) -> None:
             # Collect all the data we need to add, before adding the data back
             # to the main queue of and clearing the processing queues
             # atomically, so if this crashes, we don't lose jobs
@@ -79,10 +86,10 @@ class ReliableRedisBackend(BaseBackend):
             *processing_queue_keys,
         )
 
-    def enqueue(self, job, queue):
+    def enqueue(self, job: Job, queue: QueueName) -> None:
         self.client.lpush(self._key(queue), job.to_json().encode('utf-8'))
 
-    def dequeue(self, queue, worker_number, timeout):
+    def dequeue(self, queue: QueueName, worker_number: WorkerNumber, timeout: int) -> Optional[Job]:
         main_queue_key = self._key(queue)
         processing_queue_key = self._processing_key(queue, worker_number)
 
@@ -104,7 +111,9 @@ class ReliableRedisBackend(BaseBackend):
         if data:
             return Job.from_json(data.decode('utf-8'))
 
-    def processed_job(self, queue, worker_number, job):
+        return None
+
+    def processed_job(self, queue: QueueName, worker_number: WorkerNumber, job: Job) -> None:
         data = job.to_json().encode('utf-8')
 
         self.client.lrem(
@@ -113,10 +122,15 @@ class ReliableRedisBackend(BaseBackend):
             value=data,
         )
 
-    def length(self, queue):
+    def length(self, queue: QueueName) -> int:
         return self.client.llen(self._key(queue))
 
-    def deduplicate(self, queue, *, progress_logger=NULL_PROGRESS_LOGGER):
+    def deduplicate(
+        self,
+        queue: QueueName,
+        *,
+        progress_logger: ProgressLogger = NULL_PROGRESS_LOGGER
+    ) -> Tuple[int, int]:
         """
         Deduplicate the given queue by comparing the jobs in a manner which
         ignores their created timestamps.
@@ -137,7 +151,7 @@ class ReliableRedisBackend(BaseBackend):
 
         # A mapping of job_identity -> list of raw_job data; the entries in the
         # latter list are ordered from newest to oldest
-        jobs = {}
+        jobs = {}  # type: Dict[str, List[bytes]]
 
         progress_logger.info("Collecting jobs")
 
@@ -163,12 +177,12 @@ class ReliableRedisBackend(BaseBackend):
 
         return original_size, self.client.llen(main_queue_key)
 
-    def _key(self, queue):
+    def _key(self, queue: QueueName) -> str:
         key = 'django_lightweight_queue:{}'.format(queue)
 
         return self._prefix_key(key)
 
-    def _processing_key(self, queue, worker_number):
+    def _processing_key(self, queue: QueueName, worker_number: WorkerNumber) -> str:
         key = 'django_lightweight_queue:{}:processing:{}'.format(
             queue,
             worker_number,
@@ -176,7 +190,7 @@ class ReliableRedisBackend(BaseBackend):
 
         return self._prefix_key(key)
 
-    def _prefix_key(self, key):
+    def _prefix_key(self, key: str) -> str:
         if app_settings.REDIS_PREFIX:
             return '{}:{}'.format(
                 app_settings.REDIS_PREFIX,
